/*=======================================================================*
 |   file name : _e_cmnt.e
 |-----------------------------------------------------------------------*
 |   function  : various macros to help with comments and stuff
 |-----------------------------------------------------------------------*
 |   author    : Gregg Tavares   http://greggman.com
 |   author    : John Alvarado   http://www.biggrub.com
 |-----------------------------------------------------------------------*
 
These macros have existed since my BRIEF days, the DOS based editor I used
before I got slickedit for Windows in 93.  I don't remember what minor
changes we've had to make to keep them working but they work fine in
VS5.0 and VS7.0

Since I don't use bookmarks that much and bookmarks in brief mode are
assigned to Alt-0 through Alt-9 I generally assign these keys to

Alt-0   gat_insert_funchead()
Alt-9   gat_insert_filehead()
Alt-8   gat_insert_ifstuff()
Alt-7   gat_insert_history()
Alt-6   gat_insertcurrent_commentline()
Alt-5   gat_switch_to_header()
Ctrl+C  gat_copy_except_if_none()
Ctrl+O  gat_cd_before_load() (also assigned to File/Open... menu item)

--Below is what they do--

gat_insert_funchead(): 

   Insert a function header for the current function in various languages.
   C/C++ has the most support.  I type a typical function prototype
   
   myclass* MyFunc (int size, void* data, mystuct* pointer)
   
   With the cursor just after the closing paren I press Alt+0.
   
   I get this:

      /*************************************************************************
                                        MyFunc                                 
       *************************************************************************
      
         SYNOPSIS
      		myclass* MyFunc (int size, void* data, mystuct* pointer)
      
         PURPOSE
        		
        
         INPUT
      		size    :
      		data    :
      		pointer :
        
         OUTPUT
      		None  
        
         EFFECTS
      		None  
        
         RETURNS
        
        
         SEE ALSO
        
        
         HISTORY
      		06/13/02 GAT: Created.
        
       * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
      
      myclass* MyFunc (int size, void* data, mystuct* pointer)
      BEGINFUNC ( MyFunc)
      {
      
      
      	RETURN 
      } ENDFUNC ( MyFunc)
   
   Note: The history section above requires the system environment variable
         INITIALS to be set
         
   The other languages are not nearly as fancy.  Generally you just type the
   name of the function and press Alt-0 and you'll get a comment put above
   the function.
   
gat_insert_filehead(): 

   Pressing Alt-9 I get a typical "top-of-file" section customized for each
   supported language.  Here is the C/C++ example for a .C/.CPP file
   (vs a .H which is different)
   
   The programmer name is from the environment variable PROGRAMMER

      /*=======================================================================*
       |   file name : foo.cpp
       |-----------------------------------------------------------------------*
       |   function  : 
       |-----------------------------------------------------------------------*
       |   author    : Gregg Tavares
       *=======================================================================*/
       
      /**************************** i n c l u d e s ****************************/
      
      #include "platform.h"
      #include "switches.h"
      #include "echidna\ensure.h"
      
      /*************************** c o n s t a n t s ***************************/
      
      
      /******************************* t y p e s *******************************/
      
      
      /************************** p r o t o t y p e s **************************/
      
      
      /***************************** g l o b a l s *****************************/
      
      
      /****************************** m a c r o s ******************************/
      
      
      /**************************** r o u t i n e s ****************************/
      

gat_insert_history():

   Inserts a line that looks like this:
   
		06/13/02 GAT: 
      
   With the current day and INITIALS from the system environment variable
   INITIALS.

gat_insertcurrent_commentline()

   You type something on a line by itself and press Alt+6 and you get that
   something centered in a line of comment characters  For example
   
      constants
   
   becomes
   
      /******************************* constants *******************************/
      
   Again like the others it's language aware so if you are in a .asm file you'll get
   
      ;******************************* constants *******************************
      
   or in perl
   
      #******************************* constants *******************************
      
gat_time():

   Inserts the current time as in
   
      3:28p
      
gat_switch_to_header():

    Attempts to switch to the corresponding header file or visa versa.
    So for example if you are editing myfile.cpp and you press Alt+5
    you'll get switched to myfile.h if there is a myfile.h in the same
    folder or visa versa, if you are editing myfile.h and you press Alt+5
    you'll get switched to myfile.cpp (or .c)
    
    the current associations are
    
    .C / .CPP  <-->  .H .HPP .H++
    .S         <-->  .I

gat_cd_before_load():

    Switches the current folder to the same folder as the file you are
    currently editing so if you are editing c:\project1\file.cpp and
    you pick open to press Ctrl+O you'll be presented with the list
    of files from C:\project1  

gat_copy_except_if_none()

    Why this isn't an option in Slickedit I don't know.  In every other
    Windows program, if nothing is selected and you press/pick COPY, nothing
    happens.  In Slickedit, "nothing" gets copied to the clipboard meaning
    whatever was in the clipboard is lost.  This is almost NEVER what I or
    anybody I know wants to happen and it's not the way any other programs
    MS Word, Notepad, etc etc etc do it.
    
    This macro fixes that.  Assign Ctrl+C etc to this macro and if nothing
    is selected you won't lose your clipboard if you accidentally press Ctrl+C

gat_enter()

    Inserts a line with the comment characters at the beginning.
    For example in C would insert " *\t" for adding lines to comments
    that look like this
                    
     /****
      *
      *
      *
      ****/

The following macros are just for your information.  You may want to modify them
to your liking.

These macros are for figuring out what kind of file the current buffer is.  If
you use different extensions for files than me you'll need to add your extensions
to these macros

   boolean gat_is_c()         true if it's a C/CPP file
   boolean gat_is_lisp()      true if it's a LISP file
   boolean gat_is_asm()       true if it's an assembly file
   boolean gat_is_perl()      true if it's perl
   boolean gat_is_include()   true if it's an include file

These are the macros the handle inserting the various file headings
and function headings for each language.  Change them to insert
your style of comments

   gat_insert_cfunchead()    Inserts the function heading comments for C/CPP
   gat_insert_gcfunchead()   Inserts the function heading comments for lisp
   gat_insert_asmfunchead()  Inserts the function heading comments for assembly files
   gat_insert_perlfunchead() Inserts the function heading comments for perl
   
   gat_insert_cfilehead()    Inserts the top of file comments for C/CPP
   gat_insert_gcfilehead()   Inserts the top of file comments for lisp
   gat_insert_asmfilehead()  Inserts the top of file comments for assembly files
   gat_insert_perlfilehead() Inserts the top of file comments for perl


 *=======================================================================*/
 
#include 'slick.sh'

const barlen=73;	// Length of comment bar without the leading '/' and closing '/'

//;=====================================================================
//;**
//;** (gat_count_char string char)
//;**
//;** returns number of 'char' characters in string 'string'
//;**
int gat_count_char(str,chr)
{
	typeless count,flag,ndx;

	count = 0;
	ndx   = 1;

	while (ndx != 0)
	{
		ndx = pos(chr,str);
		if (ndx != 0)
		{
			count++;
			str = substr(str,ndx+1);
		}
	}
	return count;
}

// ;=====================================================================
// ;**
// ;** (gat_comp_dist string newstr)
// ;**
// ;** returns the difference between the last two numbers of a string.
// ;** and return the string the last number removed in 'newstr'
// ;**
// ;** example:
// ;**    (= val (gat_comp_dist "10 20 25" str))
// ;**
// ;**  val = 5
// ;**  str = "10 20"
// ;**
int gat_comp_dist (str, var newstr)
{
	typeless ndx;

	ndx    = gat_rindex(str," ");
	newstr = substr(str,1,ndx-1);
	
	return ((int)substr(newstr,gat_rindex(newstr," ")+1)-(int)substr(str,ndx+1))
}

// ;=====================================================================
// ;**
// ;** gat_compute_tabs
// ;**
// ;** returns a string that contains the tabs settings.
// ;**
bigstring gat_compute_tabs()
{
	typeless tabstr,tempstr,newstr,xpos,spaces,done,lastdist;

	// ** get tab positions.
	tabstr = strip(p_tabs);
	if (substr(tabstr,1,2) == "1 ")
	{
	  tabstr = substr(tabstr, 3);
	}

	//** remove redundant tabs positions
	spaces = gat_count_char(tabstr," ");
	if (spaces > 1)
	{
		lastdist = gat_comp_dist(tabstr,tempstr);
		done = 0;
		while (done == 0 && spaces > 1)
		{
			if (lastdist == gat_comp_dist(tempstr, newstr))
			{
				tabstr  = tempstr;
				tempstr = newstr;
				spaces  = gat_count_char(tabstr," ");
			}
			else
			{
				done = 1;
			}
		}
	}
	else
	{
		message ("COULDN'T COMPUTE TABS");
	}
	
	return tabstr;
}
#if 0
bigstring gat_compute_tabs()
{
	return(p_tabs);
}
#endif

_command gat_space() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	keyin " "
}

bigstring gat_bufname()
{
	return(strip_filename(p_buf_name,'PD'));
}

void gat_date(var da, var mo, var yr)
{
	parse _date() with mo ' |/|\-','r' da ' |/|\-','r' yr;

	if (length(mo) < 2)
	{
		mo = "0":+mo;
	}

	if (length(da) < 2)
	{
		da = "0":+da;
	}
}

boolean gat_is_c()
{
   switch (upcase(get_extension(p_buf_name)))
   {
   case 'E':
   case 'C':
   case 'H':
   case 'CPP':
   case 'HPP':
   case 'MEL':
      return(1);
   }
   return(0);
}

boolean gat_is_lisp()
{
   switch (upcase(get_extension(p_buf_name)))
   {
   case 'GC':
   case 'GS':
   case 'CL':
   case 'EL':
      return(1);
   }
   return(0);
}

boolean gat_is_asm()
{
   switch (upcase(get_extension(p_buf_name)))
   {
   case 'INC':
   case 'A65':
   case '68K':
   case 'ASR':
   case 'DEF':
   case 'S':
   case 'I':
   case 'A':
   case 'DSM':
   case 'ASM':
   case 'TBL':
   case 'INI':
   case 'DMG':
      return(1);
   }
   return(0);
}

boolean gat_is_perl()
{
   switch (upcase(get_extension(p_buf_name)))
   {
   case 'PL':
   case 'PM':
   case 'CGI':
      return(1);
   }
   return(0);
}

boolean gat_is_include()
{
   switch (upcase(get_extension(p_buf_name)))
   {
   case 'INC':
   case 'I':
   case 'H':
   case 'HPP':
      return(1);
   }
   return(0);
}

boolean gat_is_file()
{
   return(gat_is_asm()||gat_is_c())
}

boolean jma_is_log()
{
   switch (upcase(get_extension(p_buf_name)))
   {
   case 'TXT':
      return(1);
   }
   return(0);
}

void gat_insert_line(line)
{
   typeless old_indent

   old_indent = p_indent_style;
   p_indent_style = INDENT_NONE;

   keyin line
   split_insert_line();

   p_indent_style = old_indent;
}

_command gat_enter() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   typeless old_indent

   old_indent = p_indent_style;
   p_indent_style = INDENT_NONE;

   if (gat_is_asm())
   {
      split_insert_line();
      keyin ";**\t\t"
   }
   else if (gat_is_perl())
   {
      split_insert_line();
      keyin "#**\t\t"
   }
   else
   {
      split_insert_line();
      keyin " *\t"
   }

   p_indent_style = old_indent;
}

gat_right(count)
{
   while (count>0)
   {
      right;
      count--;
   }
}

//;=====================================================================
//;**
//;** jma_numstr_instr (pattern_string,  search_string)
//;**
//;** Returns the number of occurences of pattern_string in search_string.
int jma_numstr_instr(patstr, str1)
{
	typeless	slen,numstr,i, matchstr,patlen;

	patlen = length(patstr);
	slen   = length(str1);
	numstr = 0;

	i = 1;
	while (i <= slen)
	{
		matchstr = substr(str1,i,patlen);
		if (matchstr == patstr)
		{
			numstr++;
		}
		i += patlen;
	}

	return numstr;
	
} // ** jma_numstr_instr

//;=====================================================================
//;**
//;** jma_unmatched_parens (string)
//;**		
//;**	Returns number N of unmatched parenthesis in string:
//;**		N = 0	Total of left parens is same as total of right parens.
//;** 	N < 0	Total of left parens is (-N) greater then total right parens.
//;**		N > 0	Total of right parens is N greater than total left parens.
//;** Note: Direction of parens is not considered (i.e ")(" yields 0)
//;**
int jma_unmatched_parens(str1)
{
	return (jma_numstr_instr(")",str1)-jma_numstr_instr("(",str1));
} // ;** jma_unmatched_parens

int gat_rindex(str1, charstr)
{
	typeless	i;

	i = length(str1);
	while (i > 0)
	{
		if (substr(str1,i,1) == charstr)
		{
			return i;
		}
		i--;
	}
	return 0;
}

// ;=====================================================================
// ;**
// ;** jma_rindex_set (set_string search_string)
// ;**		
// ;**	Works like rindex but searches for the rightmost occurence of any
// ;** character from set_string
// ;**
int jma_rindex_set(setstr, str1)
{
	typeless loc, setlen, i, charstr;

	setlen = length(setstr);
	loc    = 0;

	i = 1;
	while (i <= setlen && loc == 0)
	{
		charstr = substr(setstr,i,1);
		loc     = gat_rindex(str1, charstr);
		i++;
	}

	return loc;
} // ** jma_rindex_set


// ;=====================================================================
// ;**
// ;** jma_insert_cfunchead
// ;**
// ;**
_command jma_insert_cfunchead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	typeless	i,
				loc,
				openloc,
				endcol,
				synline,
				parenbal,
				startline,
				endline,
				numlines,
				userstartline,
				namestr,
				linestr,
				typestr,
				prev_varname_linenum,
				result,
				is_void_func,
				is_bool_func,
            initials,
				routine_type,
            max_colon_col,
            num_args;

	// ** Find the last line of the prototype
		_begin_line();
		userstartline = p_line;
		while (1)
		{
			get_line(linestr);
			endcol = pos(')',linestr);
			if (endcol)
			{
				search(")",'+<@'); // place cursor at ')'
				endline = p_line;
				endcol  = p_col;
				break;
			}
			up
			if (p_line == 0)
			{
				message("HIT TOP OF FILE");
				p_line = userstartline;
				return(0);
			}
		}
		_begin_line();
	
	// ** Your on the last line of a function prototype (Assumed)

	// ** Find first line of prototype by matching parenthesis.
		parenbal = 0;
		do
		{
			get_line(linestr);
			parenbal += jma_unmatched_parens(linestr);
			if (parenbal < 0)
			{
				message("BAD PARENS.");
				p_line = userstartline;
				return(0);
			}
			up
			if (p_line == 0)
			{
				message("HIT TOP OF FILE.");
				p_line = userstartline;
				return(0);
			}
		} while (parenbal != 0)
		down;


	// ** Now Im on the open paren line of the function prototype.
   // ** the type and name info is either on this line also
   // ** or possibly on the previous line.

   // Is this a line with out the type and name?
   _begin_line();
	get_line(linestr);
	linestr = strip(linestr);
   if (substr(linestr,1,1) == "(")
   {
      up(); // go up to line with type and name info.
   }

	// * startline = Current line position.
   	startline = p_line;
      search("(",'+<@');   // get to line with paren.
      parenline = p_line;
      p_line = startline;  // get back to line with name.


	// * numlines = Number of lines in prototype.
		numlines = endline - startline + 1;

	// * Extract Name of function; put it in str1
	// * copy stuff upto '(' into name
		typestr = "";
		get_line(linestr);
      if (startline == parenline)
      {
         openloc = pos("(",linestr);
         namestr = substr(linestr,1,openloc-1);
      }
      else
      {
         namestr = linestr;
      }
		namestr = strip(namestr);

	// * find white space or '*' that occurs before name if any
		loc = jma_rindex_set("* \t", namestr);

	// * if white space then stuff after whitespace is name.
		if (loc != 0)
		{
			typestr = substr(namestr,1,loc);
			namestr = substr(namestr,loc+1,length(namestr)-loc);
		}
	// * else whole string is name (no <type> in string )
      is_void_func = pos('void',typestr,1,'RI') && !pos('\*',typestr,1,'R');
      is_bool_func = (typestr == 'BOOL');

	// * block Copy the arguments with the parens
      p_line = parenline;
      _begin_line();
      if (parenline==startline) search("(",'+<@'); // place cursor at '('
		_select_char('','CI');
		p_line = endline
		p_col  = endcol
		cut();
		
	// delete the function return type and name.
      p_line = startline
      _begin_line();
		_select_char('','CI');
      _begin_line(); _end_line(); left();
      _delete_selection();

	// ** Stamp routine braces with name at end.
		_begin_line();
		keyin typestr namestr:+" ";
      if (parenline!=startline) gat_insert_line("");
		paste();
		split_insert_line();
		routine_type = "FUNC";
		if (is_void_func)	routine_type = "PROC";
		_begin_line();
		keyin "BEGIN" routine_type " (":+namestr:+")";
		split_insert_line();
	 	gat_insert_line("{");
      initials = get_env("INITIALS");
      if (is_bool_func)
      {
         if ("JMA" == initials)
         {
   		   gat_insert_line("\tBOOL fSuccess;");
            gat_insert_line("");
            gat_insert_line("");
            gat_insert_line("\tRETURN fSuccess;");
         }
         else if ("GAT" == initials)
         {
   		   gat_insert_line("\tBOOL fSuccess;");
            gat_insert_line("");
            gat_insert_line("");
            gat_insert_line("\tRETURN fSuccess;");
         }
         else if ("RNM" == initials)
         {
            gat_insert_line("");
            gat_insert_line("");
            gat_insert_line("\tRETURN SUCCESS;");
            gat_insert_line("EXCEPTION:");
            gat_insert_line("\tRETURN FAILURE;");
         }
         else
         {
   		   gat_insert_line("\tBOOL fSuccess;");
            gat_insert_line("");
            gat_insert_line("");
            gat_insert_line("\tRETURN fSuccess;");
         }
      } else if (!is_void_func)
      {
         if ("JMA" == initials)
         {
            gat_insert_line("");
            gat_insert_line("");
            gat_insert_line("\tRETURN ");
         }
         else if ("GAT" == initials)
         {
            gat_insert_line("");
            gat_insert_line("");
            gat_insert_line("\tRETURN ");
         }
         else if ("RNM" == initials)
         {
            gat_insert_line("");
            gat_insert_line("");
            gat_insert_line("\tRETURN SUCCESS;");
            gat_insert_line("EXCEPTION:");
            gat_insert_line("\tRETURN FAILURE;");
         }
         else
         {
            gat_insert_line("");
            gat_insert_line("");
            gat_insert_line("\tRETURN ");
         }
      }
      else
      {
         if ("JMA" == initials)
         {
            gat_insert_line("");
            gat_insert_line("");
         }
         else if ("GAT" == initials)
         {
            gat_insert_line("");
            gat_insert_line("");
         }
         else if ("RNM" == initials)
         {
            gat_insert_line("");
            gat_insert_line("");
         }
         else
         {
   		   gat_insert_line("");
   		   gat_insert_line("");
         }
      }
		keyin "} END" routine_type " (":+namestr:+")";


	// * Go back to top of prototype
		p_line = startline;
		_begin_line();

		gat_insert_line("/":+repeat_str('*',barlen));
		keyin center(namestr,barlen); gat_insert_line("");
		gat_insert_line(" ":+repeat_str('*',barlen));
		gat_insert_line("");
		gat_insert_line("   SYNOPSIS");

	// ** Block Paste function prototype.
		synline = p_line;
		keyin typestr:+namestr:+" ";
      if (parenline!=startline) gat_insert_line("");
		paste();

		gat_insert_line("");
	// ** put "\t\t" in front of each line (numlines)of the prototype
		p_line = synline;

		for (i = 1; i <= numlines; i++)
		{
			_begin_line();
			keyin "\t\t";
			down
		}

		_begin_line();
		gat_insert_line("");
		gat_insert_line("   PURPOSE");
		keyin "  \t\t";
		_save_pos2(p);
		gat_insert_line("");
		gat_insert_line("  ");
		gat_insert_line("   INPUT");
		paste();

		search('(','-');	// get to start of parenthesized arg list

		// Format the argument names under the INPUT section.
		prev_varname_linenum = p_line-1;
      max_colon_col = 0;
      num_args = 0;
		do {
			typeless VarNameFound, CloseParenFound;
			result = search('({\([ \t]@void[ \t]@\)})|((\(|,)[ \t]@)|(([ \t]@const[ \t]@)@[ \t]@:v[ \t]@\*@[ \t]@{:v})|({[ \t]@\)})','R');
			if (result == 0)
			{
				_select_match('');
				VoidFuncFound = match_length('0');
				VarNameFound = match_length('1');
				CloseParenFound = match_length('2');
				if (VarNameFound)
				{
					search_replace('\t\t#1 :');  // replace entire match with just the variable name tabbed followed by ':'
					deselect();
					// Delete blank lines above this var name.
					up;
					while (p_line > prev_varname_linenum)
					{
						delete_line();
						up;
					}
					down;
					prev_varname_linenum = p_line;
					_begin_line(); search(':','>');
               if (p_col > max_colon_col) max_colon_col = p_col;
				 	gat_insert_line("");
               num_args++;
				}
				else if (VoidFuncFound)
				{
					_delete_selection();
				 	gat_insert_line("\t\tNone");
					break;
				}
				{
					_delete_selection();
				}
				if (CloseParenFound)
				{
					// Delete blank lines above the ')'.
					up;
					while (p_line > prev_varname_linenum)
					{
						delete_line();
						up;
					}
					down;
					break;
				}
			}
		} while (result == 0);

      /* Go back and make all the colons line up */
      if (num_args)
      {
         search("INPUT",'-');
         down();
         _begin_line();
         max_colon_col -= 1;
         for (i=0; i < num_args; i++)
         {
            search (":");
            while (p_col < max_colon_col)
            {
               keyin " ";
            }
            down();
            _begin_line();
         }
      }
		gat_insert_line("  ");

		gat_insert_line("   OUTPUT");
		gat_insert_line("\t\tNone  ");
		gat_insert_line("  ");
		gat_insert_line("   EFFECTS");
		gat_insert_line("\t\tNone  ");
		gat_insert_line("  ");
      if (!is_void_func)
      {
   		gat_insert_line("   RETURNS");
   		gat_insert_line("  ");
   		gat_insert_line("  ");
      }
		gat_insert_line("   SEE ALSO");
		gat_insert_line("  ");
		gat_insert_line("  ");
		gat_insert_line("   HISTORY");
		gat_insert_history(); gat_insert_line("Created.");
		gat_insert_line("  ");
		gat_insert_line(" ":+repeat_str("* ", barlen):+((barlen&1) ? "/":"*/"));
		
		_restore_pos2(p)
}


// =====================================================================
// **
// ** gat_insert_asmfunchead
// **
// **
_command gat_insert_asmfunchead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   typeless p

	gat_insert_line(';**':+repeat_str('*',barlen-1));
   gat_insert_line(";**");
   keyin ";** "
   _save_pos2(p);
   gat_insert_line("");
   gat_insert_line(";**");
   gat_insert_line(";** PURPOSE");
   gat_insert_line(";**\t");
   gat_insert_line(";**");
   gat_insert_line(";** INPUT");
   gat_insert_line(";**\t");
   gat_insert_line(";**");
   gat_insert_line(";** REGISTER USED");
   gat_insert_line(";**\t");
   gat_insert_line(";**");
   gat_insert_line(";** EFFECTS");
   gat_insert_line(";**\t");
   gat_insert_line(";**");
   gat_insert_line(";** RETURN VALUE");
   gat_insert_line(";**\t");
   gat_insert_line(";**");
   gat_insert_line(";** LIMITS/BUGS");
   gat_insert_line(";**\t");
   gat_insert_line(";**");
   gat_insert_line(";** HISTORY");
	gat_insert_history(); gat_insert_line("Created.");
   gat_insert_line(";**");
   _restore_pos2(p);
}

// =====================================================================
// **
// ** gat_insert_perlfunchead
// **
// **
_command gat_insert_perlfunchead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   typeless p

	gat_insert_line('#  ':+repeat_str('*',barlen-1));
   gat_insert_line("#  ");
   keyin "#   "
   _save_pos2(p);
   gat_insert_line("");
   gat_insert_line("#  ");
   gat_insert_line("#   PURPOSE");
   gat_insert_line("#  \t");
   gat_insert_line("#  ");
   gat_insert_line("#   INPUT");
   gat_insert_line("#  \t");
   gat_insert_line("#  ");
   gat_insert_line("#   EFFECTS");
   gat_insert_line("#  \t");
   gat_insert_line("#  ");
   gat_insert_line("#   RETURN VALUE");
   gat_insert_line("#  \t");
   gat_insert_line("#  ");
   gat_insert_line("#   LIMITS/BUGS");
   gat_insert_line("#  \t");
   gat_insert_line("#  ");
   gat_insert_line("#   HISTORY");
	gat_insert_history(); gat_insert_line("Created.");
   gat_insert_line("#  ");
   _restore_pos2(p);
}

// =====================================================================
// **
// ** gat_insert_gcfunchead
// **
// **
_command gat_insert_gcfunchead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   typeless p

   gat_insert_line("#|");
   gat_insert_line("* FUNCTION");
   gat_insert_line("*\t");
   gat_insert_line("*");
   gat_insert_line("* DESCRIPTION");
   keyin("*\t");
   _save_pos2(p);
   gat_insert_line("");
   gat_insert_line("*");
   gat_insert_line("* INPUTS");
   gat_insert_line("*\t");
   gat_insert_line("*");
   gat_insert_line("* OUTPUTS");
   gat_insert_line("*\tnone");
   gat_insert_line("|#");
   _restore_pos2(p);
}

#if 1
_command test_get_text()
{
	typeless ch, done;

	done = FALSE;
	do {
		ch = get_text();
		switch (ch)
		{
		case " ":
			delete_char();
			break;
		case "\n":
			cut_end_line();
			break;
		case "\r":
			cut_end_line();
			break;
		default:
			done = TRUE;
		}
	} while (!done);
}
#endif

// ;=====================================================================
// ;**
// ;** gat_insert_funchead
// ;**
// ;**
_command gat_insert_funchead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   _begin_line();

   if (gat_is_c())
   {
      jma_insert_cfunchead();
   }
   else if (gat_is_lisp())
   {
      gat_insert_gcfunchead();
   }
   else if (gat_is_perl())
   {
      gat_insert_perlfunchead();
   }
   else
   {
      gat_insert_asmfunchead();
   }
}

// ;=====================================================================
// ;**
// ;** gat_insert_ifstuff
// ;**
// ;**
_command gat_insert_ifstuff() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   _begin_line();
	gat_insert_line("");
	gat_insert_line("#if ");
	gat_insert_line("#else");
	gat_insert_line("#error NO Support for ''");
	gat_insert_line("#endif");
	gat_insert_line("");

   up(5);
   gat_right(4)
}

typeless repeat_str(str,newlen)
{
	typeless i,newstr,newstrlen,strlen,sublen;

	strlen = length(str);
	if (strlen==0 || newlen <= 0) return "";
	newstr = "";
	lenrem = newlen;
	do {
		sublen = strlen;
		if (sublen > lenrem) sublen = lenrem;
		newstr = newstr:+substr(str,1,sublen);
		lenrem -= sublen;
	} while (lenrem);

	return newstr;
}
// ;=====================================================================
// ;**
// ;** gat_insert_commentline (string)
// ;**
// ;**
gat_insert_commentline(str1)
{
   typeless head, pst, slen, numstars, numstarsleft, numstarsright;

   if (gat_is_c())
	{
		head = "/*";
		pst  = "*/";
   }
   else if (gat_is_perl())
	{
		head = "#*";
		pst  = "*";
   }
	else
	{
		head = ";*";
		pst  = "*";
	}

	slen = length(str1);
	numstars = (barlen - 1 - 1 - slen - 1 - 1);
	numstarsleft = numstars intdiv 2;
	numstarsright = numstars - numstarsleft;
	gat_insert_line("");
	up;
	_begin_line();
	keyin head repeat_str('*',numstarsleft) ' ' str1 ' ' repeat_str('*',numstarsright) pst;
	down;
	_begin_line();
}

// ;=====================================================================
// ;**
// ;** gat_insertcurrent_commentline
// ;**
// ;**
_command gat_insertcurrent_commentline() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	typeless str1;

	_begin_line();
	get_line(str1);

	if (length(str1) < (barlen-4))
	{
		_begin_line();
		_delete_line();
		gat_insert_commentline(str1);
	}
	else
	{
		message("line too long...");
	}
}

// ;=====================================================================
// ;**
// ;** gat_insert_cfilehead
// ;**
// ;**
_command gat_insert_cfilehead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	typeless	yr,mo,da,p;

	gat_date(da,mo,yr);

	top_of_buffer();
	gat_insert_line('/*':+repeat_str('=',barlen-2):+'*');
	gat_insert_line(' |   file name : ':+lowcase(gat_bufname()));
	gat_insert_line(' |':+repeat_str('-',barlen-2):+'*');
   keyin          (' |   function  : ');
   _save_pos2(p);
   gat_insert_line('');
	gat_insert_line(' |':+repeat_str('-',barlen-2):+'*');
	gat_insert_line(' |   author    : ':+get_env("PROGRAMMER"));
	gat_insert_line(' *':+repeat_str('=',barlen-2):+'*/');
	gat_insert_line(" ");
//	gat_insert_line(" *\t     \\|///-_");
//	gat_insert_line(" *\t     \\oO///_");
//	gat_insert_line(" *\t-----w/-w------");
//	gat_insert_line(" *\t E C H I D N A");
//	gat_insert_line(" *\t---------------");
//	gat_insert_line("   HISTORY");
//	gat_insert_history(); gat_insert_line("Created.");
//	gat_insert_line("     CREATED : ":+mo:+"/":+da:+"/":+substr(yr,3,2));
//	gat_insert_line("");
//	gat_insert_line(' ':+repeat_str('*',barlen):+"/");
//	gat_insert_line("");

	if (gat_is_include())
	{
		// ;* Get define label string
		typeless labelstr;

		// ;* Truncate the extension from labelstr and append '_H'
		parse gat_bufname() with labelstr '.' .
		labelstr = upcase(labelstr:+"_H");

		gat_insert_line("#ifndef ":+labelstr);
		gat_insert_line("#define ":+labelstr);
		gat_insert_commentline("i n c l u d e s");
		gat_insert_line("");
		gat_insert_line('#include "platform.h"');
		gat_insert_line("#include \"switches.h\"");
		gat_insert_line("#include \"bgl\\ensure.h\"");
		gat_insert_line("");

		gat_insert_line("#ifdef __cplusplus");
		gat_insert_line('extern "C" {');
		gat_insert_line("#endif");
		

		gat_insert_commentline("c o n s t a n t s");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_commentline("t y p e s");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_commentline("g l o b a l s");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_commentline("m a c r o s");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_commentline("p r o t o t y p e s");
		gat_insert_line("");
		gat_insert_line("");

		gat_insert_line("#ifdef __cplusplus");
		gat_insert_line("}");
		gat_insert_line("#endif");

		gat_insert_line("#endif /* ":+labelstr:+" */");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_line("");
	}
	else
	{
		gat_insert_commentline("i n c l u d e s");
		gat_insert_line("");
		gat_insert_line('#include "platform.h"');
		gat_insert_line("#include \"switches.h\"");
		gat_insert_line("#include \"bgl\\ensure.h\"");
		gat_insert_line("");
		gat_insert_commentline("c o n s t a n t s");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_commentline("t y p e s");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_commentline("p r o t o t y p e s");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_commentline("g l o b a l s");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_commentline("m a c r o s");
		gat_insert_line("");
		gat_insert_line("");
		gat_insert_commentline("r o u t i n e s");
		gat_insert_line("");
		gat_insert_line("");
	}
	
	_restore_pos2(p);
}

// ;=====================================================================
// ;**
// ;** gat_insert_gcfilehead
// ;**
// ;**
_command gat_insert_gcfilehead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	typeless	yr,mo,da,p;

	gat_date(da,mo,yr);

	top_of_buffer();
	gat_insert_line(";-*-Lisp-*-");
   gat_insert_line("(in-package goal)");
	gat_insert_line("");
//   :+repeat_str('*',barlen));
   gat_insert_line("#|");
	gat_insert_line("* MODULE");
	gat_insert_line("*\t":+gat_bufname());
	gat_insert_line("*");
   gat_insert_line("*\tCopyright (c) ":+yr:+" Naughty Dog, Inc.");
	gat_insert_line("*");
	gat_insert_line("* DESCRIPTION");
	keyin "*\t";	_save_pos2(p);
	gat_insert_line("");
	gat_insert_line("*");
	gat_insert_line("* TABS : ":+gat_compute_tabs());
	gat_insert_line("*");
	gat_insert_line("* PROGRAMMERS");
	gat_insert_line("*\t":+get_env("INITIALS"):+" = ":+get_env("PROGRAMMER"));
	gat_insert_line("*");
	gat_insert_line("* HISTORY");
	gat_insert_history(); gat_insert_line("Created.");
	gat_insert_line("*");
	gat_insert_line("|#");

	_restore_pos2(p);
}

// ;=====================================================================
// ;**
// ;** gat_insert_asmfilehead
// ;**
// ;**
_command gat_insert_asmfilehead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	typeless yr,mo,da,p;

	gat_date(da,mo,yr);

	top_of_buffer();
	gat_insert_line(';**':+repeat_str('*',barlen-1));
	gat_insert_line(';**':+repeat_str(' ',barlen-2));
	gat_insert_line(';**':+upcase(center(gat_bufname(),barlen-2)));
	gat_insert_line(';**':+repeat_str(' ',barlen-2));
	gat_insert_line(';**':+repeat_str('*',barlen-1));
	gat_insert_line(";**");
//	gat_insert_line(';**':+center("    _--`x~  ",barlen-2));
//	gat_insert_line(';**':+center("  (@ @)/|)  ",barlen-2));
//	gat_insert_line(';**':+center("\\  uuu  /) W",barlen-2));
//	gat_insert_line(';**':+center(" +-(\\  /)--+",barlen-2));
//	gat_insert_line(';**':+center("   ( \\/)    ",barlen-2));
//	gat_insert_line(';**':+center("    (  )    ",barlen-2));
//	gat_insert_line(';**':+center("    B I G   ",barlen-2));
//	gat_insert_line(';**':+center("   G R U B  ",barlen-2));
//	gat_insert_line(';**':+center("Copyright ":+yr:+" Big Grub",barlen-2));
	gat_insert_line(';**':+center("Copyright ":+yr:+" Greggman",barlen-2));
	gat_insert_line(";**");
	gat_insert_line(";**");
	gat_insert_line(";**  DESCRIPTION");
	keyin ";**\t";
	_save_pos2(p);
	gat_insert_line("");
	gat_insert_line(";**");
	gat_insert_line(";**  PROGRAMMERS");
	gat_insert_line(";**\t":+get_env("PROGRAMMER"));
	gat_insert_line(";**");
//	gat_insert_line(";**    VERSION : 00.000");
	gat_insert_line(";**  TABS : ":+gat_compute_tabs());
	gat_insert_line(";**");
	gat_insert_line(";**");
	gat_insert_line(";**  HISTORY");
	gat_insert_history(); gat_insert_line("Created.");
	gat_insert_line(";**");
	gat_insert_line(";**");
	_restore_pos2(p);

//	gat_insert_line(";**\t     \\|///-_");
//	gat_insert_line(";**\t     \\oO///_");
//	gat_insert_line(";**\t-----w/-w------");
//	gat_insert_line(";**\t E C H I D N A");
//	gat_insert_line(";**\t---------------");
}

// ;=====================================================================
// ;**
// ;** gat_insert_perlfilehead
// ;**
// ;**
_command gat_insert_perlfilehead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	typeless yr,mo,da,p;

	gat_date(da,mo,yr);

	top_of_buffer();
   gat_insert_line('#!/usr/bin/perl -w');
   gat_insert_line('use strict;');
   gat_insert_line('');
	gat_insert_line('#  ':+repeat_str('*',barlen-1));
	gat_insert_line('#  ':+repeat_str(' ',barlen-2));
	gat_insert_line('#  ':+upcase(center(gat_bufname(),barlen-2)));
	gat_insert_line('#  ':+repeat_str(' ',barlen-2));
	gat_insert_line('#  ':+repeat_str('*',barlen-1));
	gat_insert_line("#  ");
//	gat_insert_line('#  ':+center("    _--`x~  ",barlen-2));
//	gat_insert_line('#  ':+center("  (@ @)/|)  ",barlen-2));
//	gat_insert_line('#  ':+center("\\  uuu  /) W",barlen-2));
//	gat_insert_line('#  ':+center(" +-(\\  /)--+",barlen-2));
//	gat_insert_line('#  ':+center("   ( \\/)    ",barlen-2));
//	gat_insert_line('#  ':+center("    (  )    ",barlen-2));
//	gat_insert_line('#  ':+center("    B I G   ",barlen-2));
//	gat_insert_line('#  ':+center("   G R U B  ",barlen-2));
//	gat_insert_line('#  ':+center("Copyright ":+yr:+" Big Grub",barlen-2));
	gat_insert_line('#  ':+center("Copyright ":+yr:+" Greggman",barlen-2));
	gat_insert_line("#  ");
	gat_insert_line("#  ");
	gat_insert_line("#    DESCRIPTION");
	keyin "#  \t";
	_save_pos2(p);
	gat_insert_line("");
	gat_insert_line("#  ");
	gat_insert_line("#    PROGRAMMERS");
	gat_insert_line("#  \t":+get_env("PROGRAMMER"));
	gat_insert_line("#  ");
//	gat_insert_line("#      VERSION : 00.000");
	gat_insert_line("#    TABS : ":+gat_compute_tabs());
	gat_insert_line("#  ");
	gat_insert_line("#  ");
	gat_insert_line("#    HISTORY");
	gat_insert_history(); gat_insert_line("Created.");
	gat_insert_line("#  ");
	gat_insert_line("#  ");
	_restore_pos2(p);

//	gat_insert_line(";**\t     \\|///-_");
//	gat_insert_line(";**\t     \\oO///_");
//	gat_insert_line(";**\t-----w/-w------");
//	gat_insert_line(";**\t E C H I D N A");
//	gat_insert_line(";**\t---------------");
}


// ;=====================================================================
// ;**
// ;** gat_insert_filehead
// ;**
// ;**
_command gat_insert_filehead() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	if (gat_is_c())
	{
		gat_insert_cfilehead();
	}
   else if (gat_is_lisp())
   {
      gat_insert_gcfilehead();
   }
   else if (gat_is_perl())
   {
      gat_insert_perlfilehead();
   }
	else
	{
		gat_insert_asmfilehead();
	}
}

// ;=====================================================================
// ;**
// ;** gat_insert_history
// ;**
// ;**
_command gat_insert_history() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	typeless yr,mo,da,head,tail;

	gat_date(da,mo,yr);

	if (gat_is_asm())
	{
		// *** for assembly files
		head = ";**\t";
		tail = get_env("INITIALS"):+": ";
	}
	else if (gat_is_c())
	{
		// *** for c files (the default)
		head = "\t\t";
		tail = get_env("INITIALS"):+": ";
	}
	else if (gat_is_perl())
	{
		// *** for c files (the default)
		head = "#\t";
		tail = get_env("INITIALS"):+": ";
	}
   else if (gat_is_lisp())
   {
      typeless p;

      get_line(line);
      save_pos(p);
      up();
      get_line(line2);
      restore_pos(p);
      if (substr(line,1,1) == "*" || substr(line2,1,1) == "*")
      {
         head = "*\t";
      }
      else
      {
         head = ";*\t";
      }
      tail = get_env("INITIALS"):+": ";
   }
	else if (jma_is_log())
	{
		head = "";
		tail = "\t";
	}
	else
	{
		// *** All other text files
		head = "\t\t";
		tail = get_env("INITIALS"):+": ";
	}

	_begin_line();
	gat_insert_line("");
	up
	//gat_insert_line(head:+mo:+"/":+da:+"/":+substr(yr,3,2):+" ":+tail);
	keyin head:+mo:+"/":+da:+"/":+substr(yr,3,2):+" ":+tail;
}

_command gat_eval_line() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   typeless result;
   typeless str1;

	_begin_line();
	get_line(str1);
   eval_exp(result, str1, 10);
   _end_line();
   keyin "=";
   keyin result;
}


// ;=====================================================================
// ;**
// ;** gat_set_modified
// ;**
// ;**
_command gat_set_modified() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	typeless yr,mo,da,head,tail,cnt,str1;

	gat_date(da,mo,yr);

	message("setting modified data...");

	if (gat_is_c())
	{
		// *** for c files (the default)
		head = " *";
	}
   else if (gat_is_perl())
   {
		// *** for perl files
		head = "# ";
   }
	else
	{
		// *** for assembly files
		head = ";**";
	}

	cnt = 1;
	top_of_buffer();
	down(5);
	while (cnt <= 15)
	{
		get_line(str1);
		if (pos('MODIFIED',str1,1,'I'))
		{
			// ** change MODIFIED
			delete_end_line();
			keyin head:+"   MODIFIED : ":+mo:+"/":+da:+"/":+substr(yr,3,2)

			// ** change or add TABS
			down
			_begin_line();
			get_line(str1);
			if (pos('TABS',str1,1,'I')==0)
			{
				gat_insert_line("");
				up
				_begin_line();
				keyin head:+"       TABS : ":+gat_compute_tabs();
			}
			cnt = 100;
		}
		down
		cnt++;
	}

	if (cnt < 50)
	{
		message("MODIFIED NOT FOUND");
	}
}

_command _fnBeforeSave_gatbeforeSave() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	if (p_modify)
	{
		if (gat_is_file())
		{
			typeless	p;

			_save_pos2(p);
			gat_set_modified();
			_restore_pos2(p);
		}
	}
}

//=================================================================
//
// insert the time
//
_command gat_time() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
	typeless timestr;

	timestr = _time();
	
	keyin substr(timestr,1,length(timestr)-1);
}

_command gat_test() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
								//           11111111112222222222333333333344444444445
								//  12345678901234567890123456789012345678901234567890
//	message (pos('MODIFIED',"MODIFblakeeokMODIFIEedmodifiedpeMODIFIEDpfele",1,'I'));
//	typeless foo;
//	message (gat_comp_dist("10 20",foo));
   p_modify = 0;
	message (p_modify)
}


boolean gat_file_exists (var filename, var checkedFiles)
{
   _str date;

   if (length(checkedFiles)==0) {
      checkedFiles = strip_filename(filename,'PD');
   }
   else
   {
      checkedFiles = checkedFiles :+ "|" :+ strip_filename(filename,'PD');
   }
   date = _file_date (filename, 'B');
   if (date == '') {
      return FALSE;
   }
   return TRUE;
}

_command gat_switch_to_header() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   _str filename;
   _str checkedFiles;
   int  fLoad = FALSE;

   checkedFiles = '';

   switch (upcase(get_extension(p_buf_name)))
   {
   case 'C':
        fLoad    = TRUE;
        filename = strip_filename(p_buf_name,'E') :+ '.H';
        break;
   case 'I':
        fLoad    = TRUE;
        filename = strip_filename(p_buf_name,'E') :+ '.S';
        break;
   case 'S':
        fLoad    = TRUE;
        filename = strip_filename(p_buf_name,'E') :+ '.I';
        break;
   case 'H':
        fLoad    = TRUE;
        filename = strip_filename(p_buf_name,'E') :+ '.C';
        if (!gat_file_exists(filename, checkedFiles)) {
           filename = strip_filename(p_buf_name,'E') :+ '.CPP';
        }
        break;
   case 'CPP':
        filename = strip_filename(p_buf_name,'E') :+ '.HPP';
        if (!gat_file_exists(filename, checkedFiles)) {
           filename = strip_filename(p_buf_name,'E') :+ '.H++';
           if (!gat_file_exists(filename, checkedFiles)) {
              filename = strip_filename(p_buf_name,'E') :+ '.H';
              fLoad    = TRUE;
           }
        }
        break;
   }

   if (fLoad) {
      if (gat_file_exists(filename, checkedFiles)) {
           edit_file (filename);
      }
      else
      {
   		message ("Other File not found (" :+ checkedFiles :+ ")");
      }
   }

   return(0);

}

_command gat_cd_before_load() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   _str filepath;

   if (!_no_child_windows())
   {
      filepath = strip_filename(p_buf_name, 'EN');
      if (length(filepath))
      {
         chdir (filepath, 1);
      }
   }
   gui_open();
}

_command gat_copy_except_if_none() name_info(','MARK_ARG2|TEXT_BOX_ARG2|EDITORCTL_ARG2)
{
   if (!_isnull_selection()) {
      copy_to_clipboard ();
   }
}

#if 0
_command space_test() name_info(','VSARG2_REQUIRES_EDITORCTL)
{
   typeless p;
   save_pos(p);

   if (find_matching_paren() == 0)
   {
      typeless parenPos;
      _str key;

      save_pos(parenPos);
      while (test_event('R') == "")
      {
         restore_pos(parenPos);
         refresh ();
         delay (30, 'K');
         restore_pos(p);
         refresh ();
         delay (30, 'K');
      }
   }
   restore_pos(p);
}
#endif


